public class ClassTest {
    public java.lang.String var1;
    protected java.lang.String var2;
    java.lang.String var3;
    public ClassTest() {}
    public ClassTest( java.lang.String[] greetings ) {}
    public ClassTest( java.lang.String[] greetings, int default_val ) {}
    public java.lang.String getGreet() { return ""; }
    public java.lang.String getGreet(int number) { return ""; }
}
